@@ -58,13 +58,8 @@ static void ignore_print(int index, IGNORE_REC *rec)
 		g_string_append(options, "-regexp ");
 		if (rec->pattern == NULL)
 			g_string_append(options, "[INVALID! -pattern missing] ");
-#ifdef USE_GREGEX
 		else if (rec->preg == NULL)
 			g_string_append(options, "[INVALID!] ");
-#else
-		else if (!rec->regexp_compiled)
-			g_string_append(options, "[INVALID!] ");
-#endif
 	}
 	if (rec->fullword) g_string_append(options, "-full ");
 	if (rec->replies) g_string_append(options, "-replies ");
