@@ -30,6 +30,9 @@
 #include "special-vars.h"
 #include "settings.h"
 #include "lib-config/iconfig.h"
+#ifdef HAVE_CAPSICUM
+#include "capsicum.h"
+#endif
 
 #include "fe-windows.h"
 #include "window-items.h"
@@ -49,8 +52,6 @@ static THEME_REC *log_theme;
 static int skip_next_printtext;
 static char *log_theme_name;
 
-static int log_dir_create_mode;
-
 static char **autolog_ignore_targets;
 
 static char *log_colorizer_strip(const char *str)
@@ -453,7 +454,11 @@ static void autolog_open(SERVER_REC *server, const char *server_tag,
 		log_item_add(log, LOG_ITEM_TARGET, target, server_tag);
 
 		dir = g_path_get_dirname(log->real_fname);
+#ifdef HAVE_CAPSICUM
+		capsicum_mkdir_with_parents_wrapper(dir, log_dir_create_mode);
+#else
 		g_mkdir_with_parents(dir, log_dir_create_mode);
+#endif
 		g_free(dir);
 
 		log->temp = TRUE;
@@ -676,7 +681,6 @@ static void sig_theme_destroyed(THEME_REC *theme)
 static void read_settings(void)
 {
 	int old_autolog = autolog_level;
-	int log_file_create_mode;
 
 	g_free_not_null(autolog_path);
 	autolog_path = g_strdup(settings_get_str("autolog_path"));
@@ -704,12 +708,6 @@ static void read_settings(void)
 	log_theme = log_theme_name == NULL ? NULL :
 		theme_load(log_theme_name);
 
-	log_file_create_mode = octal2dec(settings_get_int("log_create_mode"));
-        log_dir_create_mode = log_file_create_mode;
-        if (log_file_create_mode & 0400) log_dir_create_mode |= 0100;
-        if (log_file_create_mode & 0040) log_dir_create_mode |= 0010;
-        if (log_file_create_mode & 0004) log_dir_create_mode |= 0001;
-
 	if (autolog_ignore_targets != NULL)
 		g_strfreev(autolog_ignore_targets);
 
