@@ -169,7 +169,8 @@ static void get_colors(int flags, int *fg, int *bg, int *attr)
                 if (*bg >= 0) {
 			*bg = mirc_colors[*bg % 100];
 			flags &= ~GUI_PRINT_FLAG_COLOR_24_BG;
-			if (settings_get_bool("mirc_blink_fix")) {
+			/* ignore mirc color 99 = -1 (reset) */
+			if (*bg != -1 && settings_get_bool("mirc_blink_fix")) {
 				if (*bg < 16) /* ansi bit flip :-( */
 					*bg = (*bg&8) | (*bg&4)>>2 | (*bg&2) | (*bg&1)<<2;
 				*bg = term_color256map[*bg&0xff] & 7;
