@@ -26,6 +26,7 @@ enum {
         TXT_WINDOW_INFO_SCROLL,
         TXT_WINDOW_SCROLL,
         TXT_WINDOW_SCROLL_UNKNOWN,
+	TXT_WINDOW_HIDELEVEL,
 
 	TXT_FILL_3,
 
@@ -52,6 +53,12 @@ enum {
 	TXT_PASTE_WARNING,
 	TXT_PASTE_PROMPT,
 
+	TXT_FILL_5, /* Welcome */
+
+	TXT_IRSSI_BANNER,
+	TXT_WELCOME_FIRSTTIME,
+	TXT_WELCOME_INIT_SETTINGS,
+
 	TXT_COUNT
 };
 
