@@ -69,7 +69,10 @@ static void sig_server_setup_fill_chatnet(IRC_SERVER_CONNECT_REC *conn,
 		return;
 	g_return_if_fail(IS_IRCNET(ircnet));
 
-	if (ircnet->nick != NULL) g_free_and_null(conn->alternate_nick);
+	if (ircnet->alternate_nick != NULL) {
+		g_free_and_null(conn->alternate_nick);
+		conn->alternate_nick = g_strdup(ircnet->alternate_nick);
+	}
 	if (ircnet->usermode != NULL) {
 		g_free_and_null(conn->usermode);
 		conn->usermode = g_strdup(ircnet->usermode);
@@ -113,14 +116,17 @@ static void sig_server_setup_fill_chatnet(IRC_SERVER_CONNECT_REC *conn,
 
 static void init_userinfo(void)
 {
+	unsigned int changed;
 	const char *set, *nick, *user_name, *str;
 
+	changed = 0;
 	/* check if nick/username/realname wasn't read from setup.. */
         set = settings_get_str("real_name");
 	if (set == NULL || *set == '\0') {
 		str = g_getenv("IRCNAME");
 		settings_set_str("real_name",
 				 str != NULL ? str : g_get_real_name());
+		changed |= USER_SETTINGS_REAL_NAME;
 	}
 
 	/* username */
@@ -131,6 +137,7 @@ static void init_userinfo(void)
 				 str != NULL ? str : g_get_user_name());
 
 		user_name = settings_get_str("user_name");
+		changed |= USER_SETTINGS_USER_NAME;
 	}
 
 	/* nick */
@@ -140,15 +147,20 @@ static void init_userinfo(void)
 		settings_set_str("nick", str != NULL ? str : user_name);
 
 		nick = settings_get_str("nick");
+		changed |= USER_SETTINGS_NICK;
 	}
 
 	/* host name */
         set = settings_get_str("hostname");
 	if (set == NULL || *set == '\0') {
 		str = g_getenv("IRCHOST");
-		if (str != NULL)
+		if (str != NULL) {
 			settings_set_str("hostname", str);
+			changed |= USER_SETTINGS_HOSTNAME;
+		}
 	}
+
+	signal_emit("irssi init userinfo changed", 1, GUINT_TO_POINTER(changed));
 }
 
 static void sig_server_setup_read(IRC_SERVER_SETUP_REC *rec, CONFIG_NODE *node)
